import click
import re
import PyInquirer as inq
from examples import custom_style_1

from wave_cli.services.validate_services import StringValidate
from wave_cli import COMMAND_NAME
from wave_cli.services.backend_services import Backend


@click.command(name='update', help='Update devices. Use options to filter devices')
@click.pass_context
@click.argument('args', nargs=1, default=None, required=False)
@click.option('-v', '--firmware-version', help='Firmware version', is_eager=True)
@click.option('-sn', '--serial-number', help='Device serial number', default=None, is_eager=True)
@click.option('-pn', '--part-number', help='Device part number', default=None, is_eager=True)
@click.option('-id', '--device-id', help='Device id', default=None, is_eager=True)
@click.option('-n', '--device-name', help='Device name', default=None, is_eager=True)
@click.option('-c', '--company', help='Company name', default=None, is_eager=True)
@click.option('-st', '--store', help='Store name', default=None, is_eager=True)
@click.option('-stn', '--store-number', help='Store number', default=None, is_eager=True)
@click.option('-cl', '--cluster', help='Cluster name', default=None, is_eager=True)
@click.option('-w', '--when', help='Update start time', default=None, is_eager=True)
def update(ctx, args, firmware_version, serial_number, part_number, device_id, device_name, company, store, store_number, cluster, when):
    try:
        if args == 'force':
            force_update = True
        else:
            force_update = False
        answer = inq.prompt(
            style=custom_style_1,
            questions=[
                {
                    'type': 'input',
                    'name': 'desired_version',
                    'message': 'Enter firmware version to update',
                    'validate': StringValidate,
                }
            ]
        )
        desired_version = answer['desired_version']
        if not firmware_version and \
                not serial_number and \
                not device_id and \
                not device_name and \
                not company and \
                not store and \
                not store_number and \
                not cluster:
            confirm = inq.prompt(
                style=custom_style_1,
                questions=[{
                    'type': 'confirm',
                    'name': 'confirm',
                    'default': False,
                    'message': 'Are you trying update all devices?'
                }],
            )['confirm']
            if not confirm:
                click.secho(message='\nBye!')
                exit(1)
            answer = inq.prompt(
                style=custom_style_1,
                questions=[
                    {
                        'type': 'input',
                        'name': 're_desired_version',
                        'message': 'So to confirm, type firmware version again [{0}]'.format(desired_version),
                        'validate': StringValidate,
                    }
                ]
            )
            re_desired_version = answer['re_desired_version']
            if desired_version != re_desired_version:
                click.secho('Not matched!', fg='red')
                exit(1)
        backend = Backend(ctx)
        where = {
            "firmwareVersion": firmware_version,
            "deviceId": device_id,
            "store": store,
            "storeNumber": store_number,
            "cluster": cluster,
            "company": company,
            "serialNumber": serial_number,
            "deviceName": device_name,
            "partNumber": part_number
        }
        result = backend.update_devices(desired_version=desired_version, where=where, is_forced=force_update, when=when)
        click.secho("")
        for key in result:
            row_length = 25
            splitted_key = re.findall('.[^A-Z]*', key)
            cap_splitted_key = [k.capitalize() for k in splitted_key]
            joined = ' '.join(cap_splitted_key)
            txt = "  {0}:".format(joined) + ' ' * (row_length - len(joined)) + "{0}".format(result[key])
            click.secho(txt, bold=True, fg='blue')
        click.secho()
        click.secho("Run `{0} show device` to tracking update states.".format(COMMAND_NAME))
        click.secho()
    except Exception as e:
        exit()



import math
import requests
import click
from halo import Halo

from wave_cli import VERSION, PACKAGE_NAME


class Version:
    def __init__(self, version_str):
        self.version_tuple = self._parse_version(version_str)
        pass

    def _parse_version(self, version_str):
        version = tuple(map(int, version_str.split('.')))
        if len(version) == 0:
            return [0, ]
        return version


def get_current_version():
    return Version(VERSION).version_tuple


def get_latest_version():
    pypi_url = 'https://pypi.org/pypi/{0}/json'.format(PACKAGE_NAME)
    result = requests.get(pypi_url)
    latest_version = result.json()['info']['version']
    return Version(latest_version).version_tuple


def compare():
    current_version = get_current_version()
    spinner = Halo(spinner='dots3', text_color='grey')
    try:
        spinner.start(text='Checking version')
        latest_version = get_latest_version()
        for i in range(len(current_version)):
            if latest_version[i] > current_version[i]:
                spinner.warn('out-of-date')
                cl = 60
                click.echo('\n')
                click.secho('╭' + (cl - 2) * '─' + '╮', fg='yellow')
                click.secho('│' + (cl - 2) * ' ' + '│', fg='yellow')
                avail_msg = 'Update available {} → {}'.format('.'.join(map(str, current_version)), '.'.join(map(str, latest_version)))
                click.secho('│' +
                            math.ceil((cl - 2 - len(avail_msg)) / 2) * ' ' +
                            avail_msg +
                            math.floor((cl - 2 - len(avail_msg)) / 2) * ' ' +
                            '│',  fg='yellow')
                hint_msg = click.style('Run ', fg='yellow') + \
                    click.style('pip3 install --upgrade {} '.format(PACKAGE_NAME), fg='blue') + \
                    click.style('to update', fg='yellow')
                click.secho(click.style('│', fg='yellow') +
                            math.floor((cl - 2 - 50) / 2) * ' ' +
                            hint_msg +
                            math.floor((cl - 2 - 50) / 2) * ' ' +
                            click.style('│', fg='yellow')
                            )
                click.secho('│' + (cl - 2) * ' ' + '│', fg='yellow')
                click.secho('╰' + (cl - 2) * '─' + '╯', fg='yellow')
                click.secho()
                return latest_version
            else:
                continue
        spinner.succeed('up-to-date')
        return None
    except requests.ConnectionError as e:
        spinner.fail(text='oops!! Connection Error. Make sure you are connected to Internet.')
        exit()
    except requests.Timeout as e:
        spinner.fail(text='oops!! Timeout Error')
        exit()
    except requests.RequestException as e:
        spinner.fail(text='oops!! General Error')
        exit()
    except KeyboardInterrupt:
        spinner.fail(text='someone closed the program')



import click
import re
import PyInquirer as inq
from examples import custom_style_1

from wave_cli.services.backend_services import Backend


@click.command(name='reset', help='Restart device services. Use options to filter devices')
@click.pass_context
@click.option('-v', '--firmware-version', help='Firmware version', is_eager=True)
@click.option('-sn', '--serial-number', help='Device serial number', default=None, is_eager=True)
@click.option('-pn', '--part-number', help='Device part number', default=None, is_eager=True)
@click.option('-id', '--device-id', help='Device id', default=None, is_eager=True)
@click.option('-n', '--device-name', help='Device name', default=None, is_eager=True)
@click.option('-c', '--company', help='Company name', default=None, is_eager=True)
@click.option('-st', '--store', help='Store name', default=None, is_eager=True)
@click.option('-stn', '--store-number', help='Store number', default=None, is_eager=True)
@click.option('-cl', '--cluster', help='Cluster name', default=None, is_eager=True)
def reset(ctx, firmware_version, serial_number, part_number, device_id, device_name, company, store, store_number, cluster):
    if not firmware_version and not serial_number \
            and not part_number \
            and not device_id and not device_name \
            and not company and not store \
            and not store_number and not cluster:
        click.secho("You are trying to reboot all devices because no filters are defined!", fg='yellow')
        confirm = inq.prompt(
            questions=[{
                'type': 'confirm',
                'name': 'confirm',
                'default': False,
                'message': 'Are you sure ?'
            }],
            style=custom_style_1
        )['confirm']
        if not confirm:
            exit(1)
    backend = Backend(ctx)
    query_params = {
      "firmwareVersion": firmware_version,
      "deviceId": device_id,
      "store": store,
      "storeNumber": store_number,
      "cluster": cluster,
      "company": company,
      "serialNumber": serial_number,
      "deviceName": device_name,
      "partNumber": part_number,
    }
    result = backend.reset_service(query_params)
    click.secho("")
    for key in result:
        row_length = 25
        splitted_key = re.findall('.[^A-Z]*', key)
        cap_splitted_key = [k.capitalize() for k in splitted_key]
        joined = ' '.join(cap_splitted_key)
        txt = "  {0}:".format(joined) + ' ' * (row_length - len(joined)) + "{0}".format(result[key])
        click.secho(txt, bold=True, fg='blue')
    click.secho()



import click
import sys
import os
import pyfiglet
import PyInquirer as inq
from examples import custom_style_1

dir_path = os.path.dirname(os.path.realpath(__file__))
parent_dir_path = os.path.abspath(os.path.join(dir_path, os.pardir))
sys.path.insert(0, parent_dir_path)

from wave_cli import COMMAND_NAME, PACKAGE_NAME, VERSION
from wave_cli.utils import read_config, save_config
from wave_cli.new_device_commands import new
from wave_cli.show_commands import show
from wave_cli.update_commands import update
from wave_cli.version_check import compare
from wave_cli.reset_command import reset
from wave_cli.reboot_commands import reboot
from wave_cli.delete_command import delete
from wave_cli.device_logs_commands import log
from wave_cli.services.backend_services import Backend


@click.group(name='main', help='Wavecount CLI for controlling and monitoring wavecount stuff.')
@click.pass_context
def main(ctx):
    compare()
    ctx.obj = read_config()
    pass


@click.command('sync', help='Synchronize cache.')
@click.option('-t', '--access-token', type=click.STRING, help='Your access token')
@click.pass_context
def sync(ctx, access_token, **kwarg):
    if not access_token:
        if 'access_token' not in ctx.obj:
            ctx.forward(login)
        else:
            access_token = ctx.obj['access_token']
    backend = Backend(context=ctx)
    result = backend.sync_cache()
    ctx.obj = {**ctx.obj, **result}
    save_config(ctx.obj)


@click.command('login', help='Authorize user.')
@click.option('-u', '--username', help='Your username', default=None)
@click.option('-p', '--password', help='Your password', default=None)
@click.pass_context
def login(ctx, username, password):
    try:
        if not username:
            username = inq.prompt(
                questions=[{'type': 'input', 'name': 'username', 'message': 'Enter username'}],
                style=custom_style_1
            )['username']
        if not password:
            password = inq.prompt(
                questions=[{'type': 'password', 'name': 'password', 'message': 'Enter password'}],
                style=custom_style_1
            )['password']
        backend = Backend(context=ctx)
        response = backend.login({'user': username, 'password': password})
        ctx.obj['name'] = response['name']
        ctx.obj['roles'] = response['roles']
        ctx.obj['access_token'] = response['accessToken']
        ctx.forward(sync)
        text_logo = 'wavecount cli'
        pyfiglet.print_figlet(text=text_logo, font='big', justify='center')
        click.secho()
        click.secho(bold=True, fg='blue', message='Welcome {}!'.format(response['name'].split(' ')[0]))
        click.secho()
        click.echo('Now run:' + click.style(text=' {} --help'.format(COMMAND_NAME), fg='blue'))
        click.secho()
    except Exception as e:
        exit()


@click.command('version', help='Show {0} version.'.format(PACKAGE_NAME))
@click.pass_context
def version(ctx):
    message = 'using version: {0}'.format(VERSION)
    click.secho(message, fg='bright_black')


main.add_command(sync)
main.add_command(login)
main.add_command(show)
main.add_command(new)
main.add_command(update)
main.add_command(reboot)
main.add_command(delete)
main.add_command(reset)
main.add_command(version)
main.add_command(log)

if __name__ == '__main__':
    main()



import click
import datetime

from wave_cli.services.backend_services import Backend


@click.group(name='show', add_help_option=False, short_help='Subcommands: [device, error]', help='Show commands managements')
def show():
    pass


@click.command(name='error', help='Show avtive errors.')
@click.pass_context
def error(ctx):
    backend_service = Backend(ctx)
    ef_list = backend_service.get_error_flags()
    if len(ef_list):
        row_len = 114
        block_len = round(row_len / 6)
        sep = '│'
        t_com = ' Company'
        t_sto = ' Store'
        t_dNm = ' Device'
        t_srl = ' S/N'
        t_err = ' Error'
        tz = str(datetime.datetime.utcnow().astimezone().tzinfo).split(' ')[0]
        t_dte = ' Date   [{0}]'.format(tz)
        click.secho('╭' + '─' * (row_len - 6) + '╮')
        click.secho(sep + t_com + ' ' * (block_len - len(t_com) - 4) +
                    sep + t_sto + ' ' * (block_len - len(t_sto) - 4) +
                    sep + t_dNm + ' ' * (block_len - len(t_dNm) - 4) +
                    sep + t_srl + ' ' * (block_len - len(t_srl) - 4) +
                    sep + t_err + ' ' * (block_len - len(t_err) + 6) +
                    sep + t_dte + ' ' * (block_len - len(t_dte) - 1) +
                    sep)
        click.secho('╞' + '═' * (row_len - 6) + '╡')
        prev_dId = ''
        for ef in ef_list:
            for f in ef['errorFlags']:
                if len(ef['deviceName']) < len(t_dNm + ' ' * (block_len - len(t_dNm) - 4)):
                    dNm = ' {0}'.format(ef['deviceName'])
                else:
                    dNm = ' {0}..'.format(ef['deviceName'][0:(len(t_dNm + ' ' * (block_len - len(t_dNm) - 4)) - 3)])
                if len(ef['company']) < len(t_com + ' ' * (block_len - len(t_com) - 4)):
                    com = ' {0}'.format(ef['company'])
                else:
                    com = ' {0}..'.format(ef['company'][0:(len(t_com + ' ' * (block_len - len(t_com) - 4)) - 3)])
                if len(ef['store']) < len(t_sto + ' ' * (block_len - len(t_sto) - 4)):
                    sto = ' {0}'.format(ef['store'])
                else:
                    sto = ' {0}..'.format(ef['store'][0:(len(t_sto + ' ' * (block_len - len(t_sto))) - 3)])
                if len(ef['serialNumber']) < len(t_srl + ' ' * (block_len - len(t_srl))):
                    srl = ' {0}'.format(ef['serialNumber'])
                else:
                    srl = ' {0}..'.format(ef['serialNumber'][0:(len(t_srl + ' ' * (block_len - len(t_srl) - 4)) - 3)])
                if len(f) < len(t_err + ' ' * (block_len - len(t_err) + 6)):
                    err = ' {0}'.format(f)
                else:
                    err = ' {0}..'.format(f[0:(len(t_err + ' ' * (block_len - len(t_err) + 6))) - 3])
                localeDate = datetime.datetime.fromtimestamp(ef['errorFlags'][f]).strftime('%Y-%m-%d %H:%M')
                dte = ' {0}'.format(localeDate)
                if prev_dId != ef['deviceId']:
                    click.secho(sep + com + ' ' * (block_len - len(com) - 4) +
                                sep + sto + ' ' * (block_len - len(sto) - 4) +
                                sep + dNm + ' ' * (block_len - len(dNm) - 4) +
                                sep + srl + ' ' * (block_len - len(srl) - 4) +
                                '├' + err + ' ' * (block_len - len(err) + 6) +
                                sep + dte + ' ' * (block_len - len(dte) - 1) +
                                sep)
                else:
                    if len(f) <= len(t_err + ' ' * (block_len - len(t_err) + 6)):
                        err = ' {0}'.format(f)
                    else:
                        err = ' {0}..'.format(f[0:len(t_err + ' ' * (block_len - len(t_err) + 5))])
                    click.secho(sep + ' ' * (3 * block_len + 6) +
                                '├' + err + ' ' * (block_len - len(err) + 6) +
                                sep + dte + ' ' * (block_len - len(dte) - 1) +
                                sep)
                prev_dId = ef['deviceId']
            click.secho('├' + '─' * (row_len - 6) + '┤')
        click.echo()
    else:
        click.secho('✔ NO ERROR WAS FOUND', fg='green')


@click.command(name='device', short_help='Show devices.', context_settings={"ignore_unknown_options": True})
@click.pass_context
@click.argument('args', default=None, required=False)
@click.option('-v', '--firmware-version', help='Firmware version', is_eager=True)
@click.option('-sn', '--serial-number', help='Device serial number', default=None, is_eager=True)
@click.option('-pn', '--part-number', help='Device part number', default=None, is_eager=True)
@click.option('-id', '--device-id', help='Device id', default=None, is_eager=True)
@click.option('-n', '--device-name', help='Device name', default=None, is_eager=True)
@click.option('-c', '--company', help='Company name', default=None, is_eager=True)
@click.option('-st', '--store', help='Store name', default=None, is_eager=True)
@click.option('-stn', '--store-number', help='Store number', default=None, is_eager=True)
@click.option('-cl', '--cluster', help='Cluster name', default=None, is_eager=True)
def device(ctx, args, firmware_version, serial_number, part_number, device_id, device_name, company, store, store_number, cluster):
    """
    Show devices. Use options to filter devices list where [ARGS] can be:
    `test`: test mode devices.
    `outup`: out-of-date devices [UNAVAILABLE]
    """
    backend_service = Backend(ctx)
    if args is not None and 'test' in args:
        test_mode = '1'
    else:
        test_mode = '0'
    where = {
        "testMode": test_mode,
        "firmwareVersion": firmware_version,
        "deviceId": device_id,
        "store": store,
        "storeNumber": store_number,
        "cluster": cluster,
        "company": company,
        "serialNumber": serial_number,
        "deviceName": device_name,
        "partNumber": part_number
    }
    dev_list = backend_service.get_devices_list(where)
    if dev_list and len(dev_list):
        if serial_number:
            for device in dev_list:
                com = device['company']
                store = device['store']
                dev_name = device['deviceName']
                store_number = device['storeNumber']
                sn = device['serialNumber']
                pn = device['partNumber']
                inst_name = device['installerName']
                inst_date = device['installingDate']
                firm_ver = device['firmwareVersion']
                firm_upd = device['firmwareUpdatedDate']
                firm_upd_state = device['updateState']
                uptodate = device['uptodate']
                dev_ip = device['deviceIP']
                mac_add = device['macAddress']
                door_w = device['doorWidth']
                door_h = device['deviceHeight']
                dist_from_door = device['distanceFromDoor']
                gat = ' - '.join(device['gatingParam'].split(' '))
                scen = ' - '.join(device['sceneryParam'].split(' '))
                state = ' - '.join(device['stateParam'].split(' '))
                alloc = ' - '.join(device['allocationParam'].split(' '))
                x1 = device['calibrationLine']['x1']
                y1 = device['calibrationLine']['y1']
                x2 = device['calibrationLine']['x2']
                y2 = device['calibrationLine']['y2']
                dev_id = device['deviceId']
                dev_info = backend_service.get_device_info(device_id=dev_id)
                prim_key = dev_info['symmetricKeys']['primaryKey']
                click.secho()
                click.secho('  Company:                  {0}'.format(com))
                click.secho('  Store:                    {0}'.format(store))
                click.secho('  Device:                   {0}'.format(dev_name))
                click.secho('  Store Number:             {0}'.format(store_number))
                click.secho('  Serial Number:            {0}'.format(sn))
                click.secho('  Part Number:              {0}'.format(pn))
                click.secho('  Installer:                {0} [{1}]'.format(inst_name, inst_date))
                click.secho('  Firmware Version:         {0} [{1}]'.format(
                    click.style(firm_ver, fg='yellow' if not uptodate else None, blink=not uptodate), firm_upd)
                )
                click.secho('  Update State:             {0}'.format(firm_upd_state))
                click.secho('  IP:                       {0}'.format(dev_ip))
                click.secho('  Mac Address:              {0}'.format(mac_add))
                click.secho('  Door Width:               {0}'.format(door_w))
                click.secho('  Device Height:            {0}'.format(door_h))
                click.secho('  Distance From Door:       {0}'.format(dist_from_door))
                click.secho('  Gating Params:            {0}'.format(gat))
                click.secho('  Scenery Params:           {0}'.format(scen))
                click.secho('  State Params:             {0}'.format(state))
                click.secho('  Allocation Params:        {0}'.format(alloc))
                click.secho('  Calibration Lines:        [x1: {0} , y1: {1}] - [x2: {2} , y2: {3}]'.format(str(x1), str(y1), str(x2), str(y2)))
                click.secho('  Device Id:                {0}'.format(dev_id))
                click.secho('  Primary Key:              {0}'.format(prim_key))
                click.secho()
        else:
            row_len = 90
            block_len = round(row_len / 7)
            sep = '│'
            t_nom = ' #   '
            t_com = ' Company'
            t_sto = ' Store'
            t_dNm = ' Device'
            t_srl = ' S/N'
            t_ver = ' Firmware Version'
            t_upd = ' Updating State'
            click.secho('╭' + '─' * (row_len + 21) + '╮')
            click.secho(sep + t_nom +
                        sep + t_com + ' ' * (block_len - len(t_com) + 2) +
                        sep + t_sto + ' ' * (block_len - len(t_sto) + 5) +
                        sep + t_dNm + ' ' * (block_len - len(t_dNm)) +
                        sep + t_srl + ' ' * (block_len - len(t_srl)) +
                        sep + t_ver + ' ' * (block_len - len(t_ver) + 5) +
                        sep + t_upd + ' ' * (block_len - len(t_upd) + 10) +
                        sep)
            click.secho('╞' + '═' * (row_len + 21) + '╡')
            count = 0
            for dev in dev_list:
                count += 1
                uptodate = dev['uptodate']
                if len(dev['company']) < len(t_com + ' ' * (block_len - len(t_com) + 2)):
                    company = ' {0}'.format(dev['company'])
                else:
                    company = ' {0}..'.format(dev['company'][0:(len(t_com + ' ' * (block_len - len(t_com) + 4)) - 3)])
                if len(dev['store']) < len(t_sto + ' ' * (block_len - len(t_sto) + 5)):
                    store = ' {0}'.format(dev['store'])
                else:
                    store = ' {0}..'.format(dev['store'][0:(len(t_sto + ' ' * (block_len - len(t_sto) + 5)) - 3)])

                firmware_version = dev['firmwareVersion']
                if len(firmware_version) < len(t_ver + ' ' * (block_len - len(t_ver) + 5)):
                    firmware_version = '{0}'.format(firmware_version)
                else:
                    firmware_version = '{0}..'.format(dev['firmwareVersion'][0:(len(t_ver + ' ' * (block_len - len(t_ver) + 2)) - 3)])

                update_state = dev['updateState']
                if update_state == 'successfully updated':
                    update_state = ''
                if len(update_state) < len(t_upd + ' ' * (block_len - len(t_upd) + 10)):
                    update_state = ' {0}'.format(update_state)
                else:
                    update_state = ' {0}..'.format(update_state[0:(len(t_upd + ' ' * (block_len - len(t_upd) + 10)) - 3)])

                device_name = ' {0}'.format(dev['deviceName'])
                serial_number = ' {0}'.format(dev['serialNumber'])
                click.secho(sep + ' ' + str(count) + ' ' * (4 - len(str(count))) +
                            sep + company + ' ' * (block_len - len(company) + 2) +
                            sep + store + ' ' * (block_len - len(store) + 5) +
                            sep + device_name + ' ' * (block_len - len(device_name)) +
                            sep + serial_number + ' ' * (block_len - len(serial_number)) +
                            sep + ' ' + click.style(firmware_version, fg='yellow' if not uptodate else None) +
                            ' ' * (block_len - len(firmware_version) + 4) +
                            sep + click.style(update_state) + ' ' * (block_len - len(update_state) + 10) + sep
                            )
            click.secho('╰' + '─' * (row_len + 21) + '╯')
            click.echo()
    else:
        click.secho('  NO DEVICE WAS FOUND!', fg='yellow')


show.add_command(error)
show.add_command(device)



from PyInquirer import ValidationError, Validator
from datetime import datetime


class SerialNumberValidate(Validator):
    def validate(self, document):
        if not (isinstance(document.text, str) and len(str(document.text)) >= 10):
            raise ValidationError(
                message='"{}" is not a valid serial-number. should be at least 10 characters'.format(document.text),
                cursor_position=len(document.text))
        else:
            return True


class PartNumberValidate(Validator):
    def validate(self, document):
        if not (isinstance(document.text, str) and len(str(document.text)) >= 4):
            raise ValidationError(
                message='"{}" is not a valid part-number. should be at least 40 characters'.format(document.text),
                cursor_position=len(document.text))
        else:
            return True


class DateValidate(Validator):
    def validate(self, document):
        try:
            datetime.strptime(document.text, '%Y-%m-%d')
            return True
        except ValueError:
            raise ValidationError(
                message='"{}" incorrect date format, should be <YYYY-MM-DD>'.format(document.text),
                cursor_position=len(document.text))


class StringValidate(Validator):
    def validate(self, document):
        chars_limit = 1
        if not (isinstance(document.text, str) and len(str(document.text)) >= chars_limit):
            raise ValidationError(
                message='"{}" is not a valid. should be at least {} characters'.format(document.text, chars_limit),
                cursor_position=len(document.text))
        else:
            return True


class IntValidate(Validator):
    def validate(self, document):
        digits_limit = 1
        if len(str(document.text)) < 1:
            raise ValidationError(
                message='"{}" is not valid. should be at least {} digits'.format(document.text, digits_limit),
                cursor_position=len(document.text)
            )
        try:
            int(document.text)
            return True
        except ValueError:
            raise ValidationError(
                message='"{}" is not valid. should be digit'.format(document.text),
                cursor_position=len(document.text))



import json
import os


root_dir = os.path.expanduser(os.path.join('~', '.wavecount_cli'))
filename = os.path.expanduser(root_dir + '/config.json')


def read_config():
    cfg = {}
    if not os.path.exists(filename):
        os.makedirs(root_dir, exist_ok=True)
        with open(filename, 'w+') as cfg_file:
            json.dump(cfg, cfg_file, indent=4)
    else:
        with open(filename, 'r') as cfg_file:
            cfg = json.load(cfg_file)
    return cfg


def save_config(cfg):
    with open(filename, 'w') as cfg_file:
        return json.dump(cfg, cfg_file, indent=4)



import json
import requests
from halo import Halo


class Backend:
    def __init__(self, context):
        self.ctx = context
        self.base_url = 'https://wavecountbackend.azurewebsites.net/api'
        self.access_token = self.ctx.obj['access_token'] if 'access_token' in self.ctx.obj else ''
        self.headers = {'Authorization': 'Bearer {}'.format(self.access_token)}

    def _call_api(self, path, method='get', params={}, data={}, running_message='Running', done_message='Done'):
        endpoint = self.base_url + path
        spinner = Halo(spinner='dots3', text_color='cyan').start(text=running_message)
        try:
            if method == 'post':
                res = requests.post(url=endpoint, json=data, params=params, headers=self.headers)
            elif method == 'delete':
                res = requests.delete(url=endpoint, headers=self.headers)
            elif method == 'get':
                res = requests.get(url=endpoint, params=params, headers=self.headers)
            else:
                raise BaseException('Method is not valid')
            if res.status_code == 200:
                spinner.succeed(text='voila! {}'.format(done_message))
                return res
            elif res.status_code == 204:
                spinner.succeed('voila! {}'.format(done_message))
            elif res.status_code == 401:
                spinner.fail(text='oops!! {}'.format(res.json()['error']['message']))
                exit()
            elif res.status_code == 422:
                spinner.fail(text='oops!! Unprocessable!: {}'.format(res.json()['error']['message']))
                exit()
            elif res.status_code == 400:
                spinner.fail(text='oops!! {}'.format(res.json()['error']['message']))
                exit()
            else:
                raise BaseException(res.json())
        except requests.ConnectionError as e:
            spinner.fail(text='oops!! Connection Error. Make sure you are connected to Internet.')
            exit()
        except requests.Timeout as e:
            spinner.fail(text='oops!! Timeout Error')
            exit()
        except requests.RequestException as e:
            spinner.fail(text='oops!! General Error')
            exit()
        except KeyboardInterrupt:
            spinner.fail(text='Someone closed the program')
            exit()

    def login(self, data):
        path = '/auth/login'
        result = self._call_api(path=path, method='post', data=data, running_message='Authenticating', done_message='Authenticated')
        return result.json()

    def sync_cache(self):
        path = '/v2/auth/sync'
        result = self._call_api(path=path, method='get', running_message='Syncing', done_message='Synchronized')
        return result.json()

    def register_device(self, data):
        path = '/devices'
        result = self._call_api(path=path, method='post', data=data, running_message='Registering')
        return result.json()

    def get_error_flags(self):
        path = '/errors'
        result = self._call_api(path=path, method='get', running_message='Fetching', done_message='Fetched')
        return result.json()

    def get_devices_list(self, where={}):
        params = {'where': json.dumps(where)}
        path = '/devices'
        result = self._call_api(path=path, params=params, running_message='Getting Devices List', done_message='Got Devices List')
        return result.json()

    def get_device_info(self, device_id: str):
        path = '/devices/{0}/info'.format(device_id)
        result = self._call_api(path=path, running_message='Getting Device Info', done_message='Got Device Info')
        return result.json()

    def update_devices(self, desired_version, where={}, when=None, is_forced=False):
        params = {'desiredVersion': desired_version, 'where': json.dumps(where), 'when': when, 'isForced': is_forced}
        path = '/devices/update'
        result = self._call_api(path=path, params=params, running_message='Running')
        return result.json()

    def reboot_devices(self,  where={}):
        params = {'where': json.dumps(where)}
        path = '/devices/reboot'
        result = self._call_api(path=path, params=params, running_message='Running')
        return result.json()

    def reset_service(self, where={}):
        params = {'where': json.dumps(where)}
        path = '/devices/reset'
        result = self._call_api(path=path, params=params, running_message='Running')
        return result.json()

    def delete_device(self, device_id):
        path = '/devices/{0}'.format(device_id)
        return self._call_api(path=path, method='delete', running_message='Deleting device {0}'.format(device_id))

    def request_device_logs(self, is_forced, device_id=None, serial_number=None, targets=None, period=None):
        path = '/logs/request'
        query_params = {
            'isForced': is_forced,
            'deviceId': device_id,
            'serialNumber': serial_number,
            'targets': targets,
            'period': json.dumps(period)
        }
        result = self._call_api(path=path, method='get', params=query_params, running_message='Preparing logs', done_message='Prepared')
        return result.json()

    def download_device_logs(self, blob_name):
        path = '/logs/download'
        query_patams = {'blobName': blob_name}
        result = self._call_api(path=path, params=query_patams, running_message='Downloading logs', done_message='Downloaded')
        return result.content



from setuptools import setup, find_packages

from wave_cli import VERSION, PACKAGE_NAME

with open("README.md", "r") as fh:
    long_description = fh.read()

DEPENDENCIES = [
    'requests >= 2.0.0',
    'click >= 7.1.2',
    'halo >= 0.0.30',
    'pyfiglet >= 0.8.post1',
    'pyinquirer >= 1.0.3',
    'pydash >= 4.9.2'
]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description="Wavecount cli",
    author="Navid Ahrary",
    author_email="n.ahrary@avakatan.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="ISC",
    classifiers=[
        "Programming Language :: Python :: 3",
        'Environment :: Console',
        'License :: OSI Approved :: ISC License (ISCL)',
        "Operating System :: OS Independent",
    ],
    packages=find_packages(exclude=['test*', '.vscode']),
    install_requires=DEPENDENCIES,
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        wave=wave_cli.wave:main
    ''',
)



import click
import PyInquirer as inq
from examples import custom_style_1

from wave_cli.services.backend_services import Backend


@click.command(name='delete', help='Delete devices. Use options to filter devices')
@click.pass_context
@click.option('-sn', '--serial-number', help='Device serial number')
@click.option('-id', '--device-id', help='Device id', default=None)
def delete(ctx, serial_number, device_id):
    if not serial_number and not device_id:
        serial_number = inq.prompt(
                questions=[{'type': 'input', 'name': 'serial_number', 'message': 'Enter serial_number'}],
                style=custom_style_1
            )['serial_number']
    backend = Backend(ctx)
    where = {
      "deviceId": device_id,
      "serialNumber": serial_number,
    }
    dev = backend.get_devices_list(where)
    if len(dev) == 0:
        click.secho('  DEVICE NOT FOUND!', fg='yellow')
        exit(1)
    confirm = inq.prompt(
        questions=[{
            'type': 'confirm',
            'name': 'confirm',
            'default': False,
            'message': 'Are you sure to delete the device "{0}" ?'.format(dev[0]['store'])
        }],
        style=custom_style_1
    )['confirm']
    if confirm:
        backend.delete_device(device_id=dev[0]['deviceId'])



import os
import click
import PyInquirer as inq
from examples import custom_style_1
from datetime import date, timedelta

from wave_cli.services.backend_services import Backend
from wave_cli.services.validate_services import DateValidate, SerialNumberValidate


@click.command(name='log', help='Get device log. Use options')
@click.pass_context
@click.argument('args', nargs=1, default=None, required=False)
@click.option('-sn', '--serial-number', help='Device serial-number', default=None, is_eager=True)
@click.option('-id', '--device-id', help='Device id', default=None, is_eager=True)
@click.option('-fd', '--from-date', help='From date', default=None, is_eager=True)
@click.option('-td', '--to-date', help='To date', default=None, is_eager=True)
def log(ctx, args, serial_number, device_id, from_date, to_date):
    try:
        force_get_log = False
        if args == 'force':
            force_get_log = True
        questions = []
        if not serial_number and not device_id:
            questions.append(
                {
                    'type': 'input',
                    'name': 'serial_number',
                    'message': 'Enter device "serial-number"',
                    'validate': SerialNumberValidate,
                }
            )
        if not from_date:
            questions.append(
                {
                    'type': 'input',
                    'name': 'from_date',
                    'message': 'Enter "start-date". format <YYYY-MM-DD>',
                    'default': str(date.today() - timedelta(days=1)),
                    'validate': DateValidate
                }
            )
        if not to_date:
            questions.append(
                {
                    'type': 'input',
                    'name': 'to_date',
                    'message': 'Enter "end-date". format <YYYY-MM-DD>',
                    'default': str(date.today()),
                    'validate': DateValidate
                }
            )
        questions.append(
            {
                'type': 'checkbox',
                'name': 'targets',
                'message': 'Select targets',
                'choices': [
                    {'enabled': True, 'name': 'Azure', 'value': 'azure'},
                    {'enabled': True, 'name': 'Frames', 'value': 'frames'},
                    {'enabled': True, 'name': 'Panel', 'value': 'panel'},
                    {'enabled': True, 'name': 'Sensor', 'value': 'sensor'},
                ],
                'validate': lambda answer: 'You must choose at least one tragets.' if len(answer) == 0 else True
            }
        )
        answers = inq.prompt(
            questions=questions,
            style=custom_style_1
        )
        backend = Backend(ctx)
        period = {
            'from': from_date if from_date else answers['from_date'],
            'to': to_date if to_date else answers['to_date'],
        }
        blob_data = backend.request_device_logs(
            is_forced=force_get_log,
            period=period,
            device_id=device_id,
            serial_number=serial_number if serial_number else answers['serial_number'],
            targets=answers['targets']
        )
        blob_name = blob_data['blobName']
        file = backend.download_device_logs(blob_name)
        local_path = os.path.expanduser("~/Desktop")
        local_file_path = blob_name.replace('/', '_')
        download_file_path = os.path.join(local_path, local_file_path)
        with open(download_file_path, 'wb+') as download_file:
            download_file.write(file)
        click.secho('  Saved at {}'.format(download_file_path), fg='green')
    except Exception as e:
        exit()



import pydash as _
import click
import PyInquirer as inq
from examples import custom_style_1
from pydash.collections import find

from wave_cli.utils import save_config
from wave_cli.services.backend_services import Backend
from wave_cli.services.validate_services import IntValidate, PartNumberValidate, StringValidate, SerialNumberValidate


@click.command(name='new', help='Register a new device.')
@click.pass_context
def new(ctx):
    try:
        roles: str = ctx.obj['roles']
        backend_service = Backend(context=ctx)
        result = backend_service.sync_cache()
        companies = result['companies']
        company_choices = ['new', *_.map_(companies, 'company')]
        part_numbers = result['partNumbers']
        part_numbers_choices = []
        if roles == 'admin':
            part_numbers_choices.extend(['new'])
        part_numbers_choices.extend(part_numbers)
        answer = inq.prompt(
            questions=[
                {
                    'type': 'input',
                    'name': 'serial_number',
                    'message': 'Enter device "serial number"',
                    'validate': SerialNumberValidate,
                },
                {
                    'type': 'list',
                    'name': 'part_number',
                    'message': 'Select "part number"',
                    'choices': part_numbers_choices,
                    'validate': PartNumberValidate,
                }
            ],
            style=custom_style_1
        )
        serial_number: str = answer['serial_number']
        part_number: str = answer['part_number']
        if part_number == 'new':
            answer = inq.prompt(
                style=custom_style_1,
                questions=[
                    {
                        'type': 'input',
                        'name': 'part_number',
                        'message': 'Enter device "part number"',
                        'validate': PartNumberValidate,
                    }
                ],
            )
            part_number: str = answer['part_number']
        answer = inq.prompt(
            questions=[
                {
                    'type': 'list',
                    'name': 'company',
                    'message': 'Select "company"',
                    'choices': company_choices,
                    'validate': StringValidate,
                }
            ],
            style=custom_style_1
        )
        company: str = answer['company']
        if company == 'new':
            answers = inq.prompt(
                style=custom_style_1,
                questions=[
                    {
                        'type': 'input',
                        'name': 'company',
                        'message': 'Enter "company"',
                        'validate': StringValidate,
                    },
                    {
                        'type': 'input',
                        'name': 'store',
                        'message': 'Enter "store"',
                        'validate': StringValidate,
                    },
                    {
                        'type': 'input',
                        'name': 'store_number',
                        'message': 'Enter "store number"',
                        'validate': IntValidate,
                    },
                ],
            )
            company: str = answers['company']
            store: str = answers['store']
            store_number: str = answers['store_number']
        else:
            store_choices = ['new']
            company_item = _.find(companies, {'company': company})
            stores_items = company_item['stores']
            stores = _.uniq(_.map_(stores_items, 'store'))
            store_choices.extend(stores)
            answer = inq.prompt(
                style=custom_style_1,
                questions=[
                    {
                        'type': 'list',
                        'name': 'store',
                        'message': 'Enter "store"',
                        'choices': store_choices,
                        'validate': StringValidate,
                    },
                ],
            )
            store: str = answer['store']
            if store == 'new':
                answer = inq.prompt(
                    style=custom_style_1,
                    questions=[
                        {
                            'type': 'input',
                            'name': 'store',
                            'message': 'Enter "store"',
                            'validate': StringValidate,
                        },
                        {
                            'type': 'input',
                            'name': 'store_number',
                            'message': 'Enter "store number"',
                            'validate': IntValidate,
                        },
                    ],
                )
                store: str = answer['store']
                store_number: str = answer['store_number']
            else:
                store_item = _.find(stores_items, {'store': store})
                store_number: str = store_item['storeNumber']
        data = {
            'store': store,
            'company': company,
            'storeNumber': store_number,
            'serialNumber': serial_number,
            'partNumber': part_number,
        }
        print(data)
        device = backend_service.register_device(data)
        dev_id = device['deviceId']
        prim_key = device['symmetricKey']['primaryKey']
        comp = device['company']
        store = device['store']
        store_num = device['storeNumber']
        sn = device['serialNumber']
        pn = device['partNumber']
        click.secho(' Part Number:    {}'.format(pn), fg='green')
        click.secho(' Serial Number:  {}'.format(sn), fg='green')
        click.secho(' Device Id:      {}'.format(dev_id), fg='green')
        click.secho(' Primary Key:    {}'.format(prim_key), fg='green')
        click.secho(' Company:        {}'.format(comp), fg='green')
        click.secho(' Store:          {}'.format(store), fg='green')
        click.secho(' Store Number:   {}'.format(store_num), fg='green')
        click.secho()
    except Exception as e:
        exit(1)
